#
# spec file for package read-only-root-fs
#
# Copyright (c) 2025 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


Name:           read-only-root-fs
Version:        1.0+git20250422.3e17744
Release:        1.1
Summary:        Files and Scripts for a RO root fileystem
License:        GPL-2.0-or-later
Group:          System/Fhs
URL:            https://github.com/openSUSE/read-only-root-fs
Source:         read-only-root-fs-%{version}.tar.xz
Source1:        README.packaging.txt
BuildRequires:  dracut
BuildRequires:  update-bootloader-rpm-macros
Requires:       dracut
Requires(post): /usr/bin/mv /usr/bin/rsync /usr/bin/gawk /usr/sbin/btrfs
Requires(post): snapper
# Required if system with new /etc/fstab entries is supposed to be updated
Conflicts:      transactional-update < 5.0.0
# Support for /etc as subvolume
Conflicts:      combustion < 1.5
Conflicts:      ignition < 2.21.0
BuildArch:      noarch
%{update_bootloader_requires}

%description
Files, scripts and directories to run the system with a
read-only root filesystem with nested writable %{_sysconfdir} BTRFS subvolume.

This package should never be installed in an already running
system! It should only be selected by a system role for a
read-only root filesystem with transactional updates.
The package will create / modify entries for mounting /etc and /var.
Those entries are used by dracut to mount the overlay file systems
during the early boot phase.

%package volatile
Summary:        Dracut Module to mount a tmpfs overlay on a RO root
Group:          System/Fhs
Requires:       dracut

%description volatile
A dracut module which mounts an overlayfs each on /etc and /var, with the upper
layer in a tmpfs mount. This is the minimal setup to get a booting system, to
have a writable /root or /home, additional fstab entries can be added.

%prep
%autosetup -p1

%build

%install
cp -a etc usr %{buildroot}

%post
if [ "$1" = 1 -a "`findmnt -n -o FSTYPE -l /`" = "btrfs" ] ; then
    %{_libexecdir}/setup-etc-subvol
fi
if [ ! -e /boot/writable -a "`findmnt -n -o FSTYPE -l /`" = "btrfs" ]; then
    %{_sbindir}/mksubvolume /boot/writable
fi
%{?update_bootloader_refresh_post}

%posttrans
if [ -f %{_sysconfdir}/zypp/zypp.conf ]; then
    sed -i 's/^multiversion =.*/multiversion =/g' %{_sysconfdir}/zypp/zypp.conf
fi
# btrfsmaintainence uses as default "/", but that is read-only.
# Change that to /.snapshots/
for var in BTRFS_BALANCE_MOUNTPOINTS BTRFS_SCRUB_MOUNTPOINTS BTRFS_TRIM_MOUNTPOINTS; do
    grep -q "${var}=\"/\"" /etc/sysconfig/btrfsmaintenance && sed -i "s|${var}=.*|${var}=\"/.snapshots\"|g" /etc/sysconfig/btrfsmaintenance
done
%{?update_bootloader_posttrans}
exit 0

%files
%license COPYING
%{_libexecdir}/setup-etc-subvol
%{_prefix}/lib/dracut/dracut.conf.d/10-read-only-root-fs.conf
%dir %{_prefix}/lib/dracut/modules.d/50writable-etc
%{_prefix}/lib/dracut/modules.d/50writable-etc/*
%{_prefix}/lib/systemd/system-preset/*
%dir %{_prefix}/lib/systemd/system/systemd-udevd.service.d
%{_prefix}/lib/systemd/system/systemd-udevd.service.d/etcmount.conf
%dir %{_prefix}/lib/systemd/system/systemd-journal-flush.service.d
%{_prefix}/lib/systemd/system/systemd-journal-flush.service.d/afterlocalfs.conf
%dir %{_prefix}/lib/systemd/system/systemd-remount-fs.service.d
%{_prefix}/lib/systemd/system/systemd-remount-fs.service.d/writableagain.conf
%dir %{_sysconfdir}/grub.d
%config(noreplace) %{_sysconfdir}/grub.d/01_suse_ro_root

%files volatile
%license COPYING
%{_prefix}/lib/dracut/modules.d/99volatile-overlay/

%changelog
* Tue Apr 22 2025 Fabian Vogt <fvogt@suse.com>
- Update to version 1.0+git20250422.3e17744:
  * Remount /sysroot/etc between sysroot-etc.mount and initrd-fs.target
* Tue Apr 15 2025 Fabian Vogt <fvogt@suse.com>
- Update to version 1.0+git20250415.7e7aea4:
  * Add missing dependency on mountpoint
* Tue Apr 15 2025 Fabian Vogt <fvogt@suse.com>
- Add missing dependencies for %%post
* Mon Apr 14 2025 fvogt@suse.com
- Update to version 1.0+git20250414.6ef7163:
  * Add a note why we need to keep 10-read-only-root-fs.conf around
  * Migrate from /etc overlays to subvolumes
- Switch _service to use mode="manual"
* Wed Feb 28 2024 fvogt@suse.com
- Update to version 1.0+git20240228.d85232a:
  * etc/grub.d/01_suse_ro_root: Don't btrfs-mount-subvol inside grub-emu
* Mon Aug  8 2022 fvogt@suse.com
- Update to version 1.0+git20220808.cd59f4f:
  * Fix writableagain.conf
* Fri Aug  5 2022 fvogt@suse.com
- Update to version 1.0+git20220805.4a3d850:
  * Work around read-only state of subvolumes in a different way
* Mon Aug  1 2022 iforster@suse.com
- Update to version 1.0+git20220801.cbb90bc:
  * Add another workaround for read-only subvolumes (boo#1202000)
  * Correctly declare mount-overlay.sh as Bash file
- Update source service URL
* Wed Sep  2 2020 Thorsten Kukuk <kukuk@suse.com>
- Adjust btrfsmaintenance sysconfig to not use the read-only
  root filesystem [bsc#1176052]
* Thu Jul 30 2020 iforster@suse.com
- Update to version 1.0+git20200730.1243fd0:
  * Add comment about the mounting of /root in the initrd
  * Better check for already existing /etc overlay [boo#1174733]
* Tue Jul 21 2020 Thorsten Kukuk <kukuk@suse.com>
- Use file requires, add sed
* Tue Jan 21 2020 iforster@suse.com
- Update to version 1.0+git20200121.5ed8d15:
  * Always mount /root during initrd [boo#1161264]
* Tue Dec  3 2019 opensuse-packaging@opensuse.org
- Update to version 1.0+git20191203.3f7cc07:
  * Workaround /var being RO during systemd-journal-flush (boo#1156421)
* Tue Nov 12 2019 kukuk@suse.com
- Update to version 1.0+git20191112.42add9e:
  * Add RequiresMountsFor=/etc override for systemd-udevd.service
* Fri Jun  7 2019 iforster@suse.com
- Update to version 1.0+git20190607.11f8587:
  * Determine /@ programmatically
* Thu Jun  6 2019 iforster@suse.com
- Update to version 1.0+git20190606.e1f756c:
  * Mount writable /boot subvolume in GRUB environment
- Create /boot/writable subvolume to provide a place to store the
  GRUB environment block or Ignition's firstboot flag on a ro root
  device
* Wed Feb  6 2019 iforster@suse.com
- Update description to reflect new behaviour
- Fix creation of overlay directories
* Wed Feb  6 2019 kukuk@suse.de
- Fix file list
* Wed Feb  6 2019 fvogt@suse.com
- Update to version 1.0+git20190206.586e9f1:
  * Remove migration code, only handle the initial setup here
  * Drop use of fstab.sys
  * Use a dracut config file instead of a module
* Wed Jun  6 2018 kukuk@suse.de
- Update to version 1.0+git20180606.046b099:
  * Fix missing shebang
  * Make scripts executeable
  * Enable create-dirs-from-rpmdb.service by default
* Wed Apr 18 2018 fvogt@suse.com
- Update to version 1.0+git20180418.4373cf9:
  * Fix 99volatile-overlay
  * proper shell escaping in mount-overlay.sh (#3)
* Mon Apr 16 2018 fvogt@suse.com
- Update to version 1.0+git20180416.825ad6c:
  * Add new 99volatile-overlay dracut module
- BuildRequire/Require dracut for proper directory ownership
- Fix posttrans if zypper is not installed
* Wed Mar 21 2018 kukuk@suse.de
- Disable multiversion feature of libzypp at install time
- Document, that this package should never be installed in a
  running system, only during installation
* Wed Mar 21 2018 kukuk@suse.de
- Update to version 1.0+git20180321.a5b34ab:
  * Disable purge-kernels.service, does not work with read-only
    root filesystem
  * Enable man-db-create.service, man cannot create cache in %%post
  * Fix OBS URL
* Fri Mar  2 2018 rbrown@suse.com
- Add Requires(post) for coreutils & grep to ensure post script has
  what it needs (boo#1083526)
* Tue Feb 27 2018 kukuk@suse.de
- Update to version 1.0+git20180227.44d5e8a:
  * Initial import of read-only-root-fs stuff from caasp-tools
* Tue Feb 27 2018 kukuk@suse.de
- Initial version
